/*
 *	Copyright (c) Vanderbilt University, 2001-2002
 *	ALL RIGHTS RESERVED
 *
 *	Vanderbilt University disclaims all warranties with regard to this
 *	software, including all implied warranties of merchantability
 *	and fitness.  In no event shall Vanderbilt University be liable for
 *	any special, indirect or consequential damages or any damages
 *	whatsoever resulting from loss of use, data or profits, whether
 *	in an action of contract, negligence or other tortious action,
 *	arising out of or in connection with the use or performance of
 *	this software.
 *
 *  FILE NAME:	PatternMatcher.h 
 *	SYNOPSIS:	Interface for Pattern Match
 *  AUTHOR:		Feng Shi - feng.shi@vanderbilt.edu
 *              Tihamr Levendovszky - tihamer.levendovszky@vanderbilt.edu
 *  CREATED:	06/11/02
 *  LAST UPDATE: 
 *  CHANGE LOG:
 */
//////////////////////////////////////////////////////////////////////
#ifndef PATTERNMATCHER_H
#define PATTERNMATCHER_H


#include "GR.h"
#include "UdmBase.h"
#include "Effector.h"
#include "GReATTypes.h"




class PatternMatcher 
{
public:
	PatternMatcher(const GR::RewritingRule& rewritingRule, GeneralBackend *dataNetworkMap)
		: thisRule(rewritingRule),backendMap(dataNetworkMap){};	
	~PatternMatcher();

	void findMatch();
	MatchVector matches;

protected:
	bool init();
	bool reqRunMatch();
	bool getEdgeToBound(GR::ObjectWrapper &boundPO,GR::ObjectWrapper &unboundPO,\
						GR::ObjectLink &unboundPl);
	bool bindParallelEdges(GR::ObjectWrapper &poNew);
	bool reqBindEdgeList(set<GR::ObjectLink>& plList,GR::ObjectWrapper& poNew);
	bool isValidLink(GR::ObjectLink& objLink,Object& obSrc,Object& obDst);

	bool isBound(const Object& object, const string& metaName) const;
	bool isBound(const Object& object) const;

	bool isBound(const GR::ObjectLink& objLink) const;
	void bind(GR::ObjectWrapper& grObj,const Object& object);
	void bind(GR::ObjectLink& objLink);
	void unBind(GR::ObjectWrapper& grObj,Object& object);
	void unBind(GR::ObjectLink& grLink);
	bool executeGuard();
	void invokeCint();
	bool checkNegative(const GR::ObjectWrapper& grObj, Object& object) const;
	string getUId(const string& metaName, const Object& obj) const;

	bool isAllBound();

	const GR::RewritingRule& thisRule;
	GeneralBackend* backendMap; 
	Match currentMatch;	
};

#endif //PATTERNMATCHER_H
