#include "CIntCompiler.h"
#include <algorithm> // for 'for_each'
#include "PEStringTable.h"
#include "GReATUtils.h"
#include "Logging.h"

bool CIntCompiler::isGuard() const
{
	const std::string& guardExpression= ( std::string) _rule->guard();
	return ( !guardExpression.empty());
}

bool CIntCompiler::isCode() const
{
	set< GR::Code> codes= _rule->Code_kind_children();
	return ( !codes.empty());
}

/*
const std::string CIntCompiler::createGuard( PEComposite& code)
{
	_currCode= &code;
	buildOWNameMap( true);
	passObjects();
	const std::string guardVarName= createGuardInit();
	const std::string& guardExpression= ( std::string) _rule.guard();
	_currCode->add( *new Expression( guardExpression));
	return guardVarName;
}
*/

const std::string CIntCompiler::createGuard( PEComposite& code)
{	
	//
	Function* guardFuncDef= new Function;
	guardFuncDef->_name= "isGuardTrue";
	guardFuncDef->_returnValue= BOL;
	//
	FunctionCall* guardFuncCall= new FunctionCall;
	guardFuncCall->_func= guardFuncDef;
	//
	buildOWNameMap( true);
	passParameters( *guardFuncDef, *guardFuncCall);
	//
	_currCode= guardFuncDef;
	const std::string guardVarName= createGuardInit();
	const std::string& guardExpression= ( std::string) _rule->guard();
	_currCode->add( *new Expression( guardExpression));
	_currCode->add( *new Expression( RET+ SEP+ guardVarName));
	//
	guardFuncCall->_retType= BOL;
	guardFuncCall->_retVar= guardVarName;
	_addGuardFuncDefTo->addProtected( *guardFuncDef);
	code.add( *guardFuncCall);
	//
	return guardVarName;
}

// Adds 'code' to '_output'
// This is an adoptable functor.
class AddCodeFunctor : public std::unary_function< GR::Code, void>
{
public:
	AddCodeFunctor( PEComposite* output) : _output( output) {}
	void operator()( const GR::Code& code)
	{
		_output->add( *new Expression( "{"+ ( std::string) code.code()+ "}"));
	}
private:
	PEComposite* _output;
};

void CIntCompiler::createCode( PEComposite& code)
{
	_currCode= &code;
	buildOWNameMap( false);
	passObjects();
	set< GR::Code> codes= _rule->Code_kind_children();
	for_each( codes.begin(), codes.end(), AddCodeFunctor( _currCode));
}

const std::string CIntCompiler::createGuardInit() 
{
	// print 'bool Gz_guard= true;'
//	Definition* guard= new Definition;
//	guard->_type= BOL;
//	guard->_variable= "Gz_guard";
//	guard->_initValue= FAL;
	_currCode->add( *new Expression( "bool Gz_guard= false"));
	// print 'bool* guardVal= &guard;'
//	Definition* guardVal= new Definition;
//	guardVal->_type= BOL+ PTR;
//	guardVal->_variable= "guardVal";
//	guardVal->_initValue= REF+ guard->_variable;
//	_currCode->add( *guardVal);
	// return with guard variable name.
	return "Gz_guard";
}

void CIntCompiler::buildOWNameMap( bool onlyBoundObjects)
{
	std::set< GR::ObjectWrapper> OWs= _rule->ObjectWrapper_kind_children();
	for( set< GR::ObjectWrapper>::const_iterator itOWs= OWs.begin(); itOWs!= OWs.end(); ++itOWs)
	{
		const GR::ObjectWrapper& currOW= *itOWs;
		if ( onlyBoundObjects && ( "CreateNew" == ( std::string) currOW.action()))
			continue;
		OWNameMap_t::iterator itFindName= _owNameMap.find( currOW.name());
		if ( _owNameMap.end() != itFindName)
		{
			printWarning( "Object name: "+ ( std::string) currOW.name()+ " is not unique. Undeterministic AttributeMapping and/or Guard.");
			itFindName->second= std::make_pair( itFindName->second.first, false);	// set to not unique
		}
		else
			_owNameMap[ ( std::string) currOW.name()]= std::make_pair( currOW, true);	// unique by default
	}
}

void CIntCompiler::passObjects()
{
	for( OWNameMap_t::const_iterator itNames= _owNameMap.begin(); itNames!= _owNameMap.end(); ++itNames)
	{
		if ( true == itNames->second.second)	// if pattern object name is unique..
		{
			const GR::ObjectWrapper& currOW= itNames->second.first;
			std::map< GR::ObjectWrapper, std::string>::const_iterator itBoundObject= _boundObjects->find( currOW);
			if ( _boundObjects->end()== itBoundObject)
			{
				throw CGEx("FATAL: CG internal error. Pattern object: "+ ( std::string) currOW.name()+ " not bound in rule:" + ( std::string)_rule->name()+ " (CInt compiler.)");
			}
			const std::string& generatedName= (*itBoundObject).second;
			Expression* pseudo= new Expression;	// use expression, because Definition will lowercast the variable name.
			pseudo->_expression= ( std::string) currOW.paradigmName()+ RES+ ( std::string) currOW.classType()+ REF +SEP+ ( std::string) currOW.name()+ EQ+ SEP+ generatedName;
			_currCode->add( *pseudo);
		}
	}
}

void CIntCompiler::passParameters( Function& guardFuncDef, FunctionCall& guardFuncCall)
{
	for( OWNameMap_t::const_iterator itNames= _owNameMap.begin(); itNames!= _owNameMap.end(); ++itNames)
	{
		if ( true == itNames->second.second)	// if pattern object name is unique..
		{
			const GR::ObjectWrapper& currOW= itNames->second.first;
			std::map< GR::ObjectWrapper, std::string>::const_iterator itBoundObject= _boundObjects->find( currOW);
			if ( _boundObjects->end()== itBoundObject)
			{
				throw CGEx("FATAL: CG internal error. Pattern object: "+ ( std::string) currOW.name()+ " not bound in rule:" + ( std::string)_rule->name()+ " (CInt compiler.)");
			}
			//std::string paramType= CST+ SEP+ "Udm"+ RES+ "Object"+ REF;
			std::string paramType= CST+ SEP+ ( std::string)currOW.paradigmName()+ RES+ ( std::string)currOW.classType()+ REF;
			const std::string& paramName= currOW.name();
			const std::string& generatedName= (*itBoundObject).second;
			guardFuncDef.insertInputParam( FuncParam_t( make_pair( paramType, paramName)));
			guardFuncCall._params.push_back( generatedName);
		}
	}
}

