/* NAME:	Sequencer
 * AUTHOR:	Attila Vizhanyo - viza@isis-server.isis.vanderbilt.edu
 *
 * Iterates GE data structure and call the appropriate translator code 
 */
#ifndef __SEQUENCER_H
#define __SEQUENCER_H

#ifdef _MSC_VER
#pragma warning (disable : 4786)
#pragma warning (disable : 4503) 
#endif

#include <comdef.h>
#include <list>
#include <stack>
#include "UmlExt.h"
#include "GR.h"
#include "RuleTypes.h"
#include "PE.h"
#include "Translator.h"

class Sequencer
{
public:
	// ctor & dtor
	Sequencer( Translator& translator);
	virtual ~Sequencer();
	// Main sequencer function.
	void execute( const GR::RuleBase& rule);
	// Sequencer for different types of GR rules.
	void executeProxy( const GR::RewritingRuleProxy& proxy);
	void executeRewritingRule( const GR::RewritingRule& rule);
	void executeBlock( const GRBlock& rule);
	void executeForBlock( const GRForBlock& rule);
	void executeTest( const GRTest& rule);
	void executeForTest( const GRForTest& rule);
	void executeRule( const GRRule& rule);
	void executeCase( const GRCase& rule);

protected:
	// Create std::stack of destination rules 'dstRules', to which 'rule' is connected as source.
	void destinationRules( const GR::RuleBase& rule, const GR::RuleBase& parent, std::stack< GR::RuleBase>& dstRules) const;
	// Returns true, if 'src' is (possibly indirect) source of 'dst'. Used for loop detection.
	bool isSource( const GR::RuleBase& src, const GR::RuleBase& dst) const;
	// Translates calls of all rules inside 'block'. 
	void translateAllCalls( const GRBlockType& block);
	// Performs in-breadth traverse on sequences 'currSeqs'.
	void traverseSequences( const std::list< GR::Sequence>& currSeqs);
	// Translate those 'seqs' which had been not traversed before, mark them traversed.
	void translateSequences( const std::list< GR::Sequence>& seqs);
	// Returns true, if 'rule' has already been traversed
	bool isRuleTraversed( const GR::RewritingRule& rule) const;
	// Set 'rule' traversed, i.e. put object into '_traversedObjects'
	void setRuleTraversed( const GR::RewritingRule& rule);
	// Sequence traverse maintenance.
	bool isSeqTraversed( const GR::Sequence& seq) const;
	void setSeqTraversed( const GR::Sequence& seq);

private:
	// Translator member does the GR to PE translation. (See Translator.h)
	Translator& _translator;
	// Keep track of traversed rules. (Used by the 'execute...()' function family.)
	typedef std::set< GR::RewritingRule> RuleSet_t;
	RuleSet_t _traversedRules;
	// Keep track of traversed sequences. (Used by 'translateAllCalls()'.)
	typedef std::set< GR::Sequence> TraversedSeqs_t;
	TraversedSeqs_t _traversedSeqs;
	// Solemnly used for logging rule hierarchies.
	int _depth;
};
 

#endif	//__SEQUENCER_H

