#ifndef GReATDebuggerVars_H
#define GReATDebuggerVars_H

#include "GReATTypes.h"
#include "..\GRDebugInterface\GRDebugVariable.h"

typedef std::list<Udm::Object> UdmObjectList;

namespace GRDI
{
//=========================================
class IFunctionImpl
{
protected:

  GR::RuleBase m_rule;
  GR::RuleBase m_ref; 

public:
    int m_match;
public:
  IFunctionImpl(const GR::RuleBase& rule);
  virtual ~IFunctionImpl() {}
  std::string getFullPath() const;
};
//=========================================
class IVariableImpl
{
public:

  typedef enum {INPUT, OUTPUT, IO, PATTERN} Type; 
  Type m_type;
public:
  IVariableImpl():m_type(INPUT){}
  virtual ~IVariableImpl() {}
};
//=========================================

class IVariableValueImpl
{
protected:

  UdmObjectList m_objList;

public:
  IVariableValueImpl(){}
  void addValue(const Udm::Object& obj) {m_objList.push_back(obj);}
  const UdmObjectList& getObjList() const {return m_objList;}
  virtual ~IVariableValueImpl() {}
};
};
//=========================================
void getObjPath(const Udm::Object& obj, std::string& str);
//=========================================
class GRFunction : public GRDI::IFunctionImpl,
                   public GRDI::IFunction
                   
{

public:
  GRFunction(const GR::RuleBase& rule);
  const GRDI::FunctionID getID() const;
  void print(std::ostream& stream, int indent = 0) const;
  GRDI::IFunctionImpl* getIFunctionImpl();
};

//=========================================
class GRVariable : public GRDI::IVariableImpl,
                   public GRDI::IVariable
                   
{
protected:
  GR::ObjectWrapper m_var;
public:
  GRVariable(const GR::ObjectWrapper& var);
  void print(std::ostream& stream, int indent = 0) const;
  std::string name() const;
  GRDI::IVariableImpl* getIVariableImpl();
};
//=========================================
class GRVariableValue : public GRDI::IVariableValueImpl,   
                        public GRDI::IVariableValue
{
public:
   void print(std::ostream& stream, int indent = 0) const;
  GRDI::IVariableValueImpl* getIVariableValueImpl();
};

//=========================================
class GRVariableFactory
{

public:
  
  static GRDI::IFunctionPtr createFunction(const GRFunction& grf) 
   {return GRDI::IFunctionPtr(new GRFunction(grf));}

  static GRDI::IVariablePtr createVariable(const GR::ObjectWrapper& var) 
   {return GRDI::IVariablePtr(new GRVariable(var));}

  static GRDI::IVariableValuePtr createVariableValue(const GRVariableValue& var) 
   {return GRDI::IVariableValuePtr(new GRVariableValue(var));}
};

#endif //GReATDebuggerVars_H