#pragma warning( disable : 4503)

#include "StdAfx.h"

#include "GRThread.h"
#include "GREngineDlgDlg.h"
#include "GReATFactory.h"
#include "GReATParametersFactory.h"
#include "CINTCodeGathererFactory.h"

//============================
void Notify::reportProgress(double prg) 
{
	if (!m_quit)
	{
		UINT nprg =  (UINT )(m_maxPrg * prg);
		m_dlg.SendMessage(SET_PROGRESS, (LPARAM) nprg);
	}
}

void Notify::reportExcuteRule(const string &rule)
{
	if (!m_quit)
	{
		CString str(rule.c_str());
		m_dlg.SendMessage(SET_EXECUTERULE, (LPARAM)(LPCTSTR) str);
	}
}

//============================
void Notify::finished() 
{
	m_dlg.PostMessage(GR_FINISHED, 0);
}

//============================
GRThread::GRThread(const std::string& fname) :
				m_fname(fname),
				m_notify(NULL)
{
}

GRThread::GRThread( const std::string& fname,
					Notify* notify) :
                 m_fname(fname),
				 m_notify(notify)
                  
{
}

GRThread::GRThread(const std::string& fname, 
				   const vector<std::string> &inputs,
				   Notify* notify) :
				m_fname(fname),
				m_inputs(inputs),
				m_notify(notify)
{
}

//============================
void GRThread::operator()()
{
	CoInitialize(0);  //Magic happens here
	IGReATParameters* grParams  = 0;
	IGReAT* great = 0;
	  ICINTCodeGatherer* cintDll = 0;

	try
	{
		grParams = GReATParametersFactory::create();
		if(m_inputs.empty())  
			grParams->open(m_fname);
		else 
			grParams->open(m_fname, m_inputs);

		GeneralBackendPtr dataNetworkMap;
		grParams->getGeneralBackend(dataNetworkMap);

		UdmDataNetworkPtr transformation;
		grParams->getTransformation(transformation);

		great = GReATFactory::create();
		great->setCtrlNotify(m_notify);
		great->init(dataNetworkMap, transformation);

/////////////////////////////////////////////////////////////
  	cintDll = CINTCodeGathererFactory::create();
    UserCodeGenConfig cfg;
    cfg.m_depFileName = grParams->getTransfLastWrittenFileName();
    cfg.m_udmDirName = grParams->getUdmDirName();
    cfg.m_transf = transformation;
    cfg.m_metas = grParams->getMetas();

    StrVec incFiles, libFiles, incDirs, libDirs;
    grParams->getUserCodeInfo(incFiles, libFiles, incDirs, libDirs);

    cfg.m_incFiles = incFiles;
    cfg.m_libFiles = libFiles;
    cfg.m_incDirs = incDirs;
    cfg.m_libDirs = libDirs;

    #ifdef _DEBUG
      cfg.m_debug = true;
    #else
      cfg.m_debug = false;
    #endif

    cintDll->generateDll(cfg); 
    HINSTANCE hDll  = cintDll->loadDll();
    great->setCINTDll(hDll);
/////////////////////////////////////////////////////////////

		GR::RewritingRule startRule;
		PassAlongPackets arguments;
		grParams->getStartRule(startRule);
		grParams->getPassAlongPackets(arguments);
		great->Execute(startRule, arguments);
		//====================
		arguments.clear();
		if (great)
			delete great, great = 0;
		if(!m_notify->m_cancel)
		{
			grParams->close();
			grParams->getOutputMsg(CGREngineDlgDlg::outputMsg);
		}
		else grParams->closeNoSave();

		if (grParams)
			delete grParams, grParams = 0;
	}
	catch(udm_exception &exc)
	{
		AfxMessageBox(CString("Exception: ") + CString(exc.what()));
		if (great)
			delete great, great = 0;
		grParams->closeNoSave();
		m_notify->m_quit = true;

		if (grParams)
		delete grParams, grParams = 0;

		if(m_notify)  m_notify->finished();

		if (cintDll)
		delete cintDll, cintDll = 0;

		CoUninitialize();

		throw exc;
	}

//	if (grParams)
//		delete grParams, grParams = 0;

	if(m_notify)  m_notify->finished();

	  if (cintDll)
    delete cintDll, cintDll = 0;
	
	CoUninitialize();
}
