#ifndef TEMPDNPROCESSOR_H
#define TEMPDNPROCESSOR_H

#include <functional>
#include <algorithm>
#include "UMLModelTransformer.h"
#include "GReATConfig.h"
#include "TempDNEx.h"

class TempDNProcessor : public std::binary_function< UMLModelTransformer::RootFolder, GReATConfig::UsageInformation, int>
{
public:
	TempDNProcessor( const std::string& udmProjectFolder) : _udmProjectFolder( udmProjectFolder) {
	}
	//
	result_type operator()( first_argument_type rt, second_argument_type ui) const {
		std::vector< UMLModelTransformer::Package> packages= rt.Package_kind_children();
		processPackages( packages, ui);
		std::vector< UMLModelTransformer::RootFolder> rootfolders= rt.RootFolder_kind_children();
		std::for_each( rootfolders.begin(), rootfolders.end(), std::bind2nd( *this, ui));
		return 0;
	}
protected:
	//
	void processPackages( const std::vector< UMLModelTransformer::Package>& ps, GReATConfig::UsageInformation& ui) const;
	//
	UMLModelTransformer::Class findRootObjectInPackage( const UMLModelTransformer::Package& p) const;
	//
	bool isReferencedByChild( const UMLModelTransformer::Class& c) const;

private:
	std::string _udmProjectFolder;
};

extern std::string getUdmProjectFolder( const UMLModelTransformer::Configuration& cfg);

#endif //TEMPDNPROCESSOR_H