#include "stdafx.h"

#include "Component.h"
#include <afxdlgs.h>
#include "GR.h"
#include <UdmGme.h>			// these must be included to make SmartDataNetwork work
#include <UdmDom.h>
#include "UMLModelTransformer2GR.h"
#include "umt_exception.h"

char* GetRelativeFilename(const char *currentDirectory,const char *absoluteFilename);

using namespace GR;

void CComponent::InvokeEx(CBuilder &builder,CBuilderObject *focus, CBuilderObjectList &selected, long param) 
{
	CBuilderFolder *root = builder.GetRootFolder();
	const CBuilderFolderList *subfolders = root->GetSubFolders();

	// Writing back information to config
	////////////////////////////////////////////////////////////////////////////////////////////
	CBuilderModel *theCfg;
	if(focus != NULL && focus->GetKindName() == "Configuration")
		theCfg = BUILDER_CAST(CBuilderModel, focus);
	else
	{	POSITION spos = subfolders->GetHeadPosition();
		CBuilderModelList configs;
		while(spos)
		{
			CBuilderFolder *subfolder = subfolders->GetNext(spos);
			if(subfolder->GetKindName() != "Configurations")
				continue;
			
			const CBuilderModelList *cfgs = subfolder->GetRootModels();
			POSITION cmpos = cfgs->GetHeadPosition();
			while(cmpos)
			{	configs.AddTail(cfgs->GetNext(cmpos));
			}
		}
		if(configs.GetCount() == 0)
		{	throw umt_exception("Generate GR Error: No Configuration model found. Use UMT Master Interpreter.");
		}
		else if(configs.GetCount() > 1)
		{	throw umt_exception("Generate GR Error: More than one Configuration model found. Either Start interpretation from one or delete the others.");
		}
		else
		{	theCfg = configs.GetHead();
		}
	}	

	CBuilderAtom *theMetaIcon;
	const CBuilderAtomList *mis = theCfg->GetAtoms("MetaInformation");
	if(!mis || mis->GetCount() == 0)
		theMetaIcon = theCfg->CreateNewAtom("MetaInformation");
	else
		theMetaIcon = mis->GetHead();
	
	CString cfgFileName;
	CString cfgDirName;
	theCfg->GetAttribute("ConfigFile",cfgFileName);
	if(cfgFileName == "" || cfgFileName.GetLength() <3 || cfgFileName.GetAt(1) != ':')
	{	throw umt_exception("Generate GR Error: Config file name is not absolute. Run UMT Master interpreter.");
	}
	
	int p = cfgFileName.ReverseFind('\\');
	cfgDirName = cfgFileName.Left(p+1);
	
	CString transFileName;
	theMetaIcon->GetAttribute("TransformationFile",transFileName);
	if(transFileName == "")
	{	throw umt_exception("Generate GR Error: Transformation (GR) file blank. Run UMT Master interpreter.");
	}
	
	CString transFilePath = cfgDirName + transFileName;
	CString transDirName;
	
	p = transFilePath.ReverseFind('\\');
	transDirName = transFilePath.Left(p+1);
	//AfxMessageBox(transDirName);
	if(CreateDirectory((LPCSTR)transDirName, NULL)==0)
	{	int le = GetLastError();
		if(le != 183)
		{	LPVOID lpMsgBuf;
			FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
							NULL, le, 0, (LPTSTR) &lpMsgBuf, 0, NULL );
			string msg = "Generate GR Error: Unable to create directory " + transDirName;
		//	msg += (LPCSTR)lpMsgBuf;
			msg += ".";
			throw umt_exception(msg);
		}
	}
	{	
		Udm::SmartDataNetwork nw(diagram);

		nw.CreateNew(string(transFilePath),"GR.XSD", RootFolder.meta);
		RootFolder rrr = RootFolder::Cast(nw.GetRootObject());
		
		POSITION pos = subfolders->GetHeadPosition();
		while(pos)
		{	CBuilderFolder *subfolder = subfolders->GetNext(pos);
			if(subfolder->GetKindName() != "Transformation")
				continue;
			FolderPhase1(subfolder, rrr);
		}
		pos = subfolders->GetHeadPosition();
		while(pos)
		{	CBuilderFolder *subfolder = subfolders->GetNext(pos);
			if(subfolder->GetKindName() != "Transformation")
				continue;
			FolderPhase2(subfolder, rrr);
		}
		
		nw.CloseWithUpdate();
	}
	/*catch(const udm_exception &e)
	{
		AfxMessageBox(e.what());
		throw e;

	}*/

	//AfxMessageBox("The Equivalent Transformation has been created" + builder.GetRootFolder()->GetName(), MB_OK | MB_ICONINFORMATION);

}

void CComponent::FolderPhase1(CBuilderFolder *trans, RootFolder root)
{
	const CBuilderModelList *expressions = trans->GetRootModels();
	POSITION pos = expressions->GetHeadPosition();
	while(pos)
	{
		UMT_ExpressionModel *expression = BUILDER_CAST(UMT_ExpressionModel, expressions->GetNext(pos));
		if(!expression)
			continue;
		expression->SetUdmParent(root);
		expression->Phase1_CreateObjects();
	}

}

void CComponent::FolderPhase2(CBuilderFolder *trans, RootFolder root)
{
	const CBuilderModelList *expressions = trans->GetRootModels();
	POSITION pos = expressions->GetHeadPosition();
	while(pos)
	{
		UMT_ExpressionModel *expression = BUILDER_CAST(UMT_ExpressionModel, expressions->GetNext(pos));
		if(!expression)
			continue;
		expression->SetUdmParent(root);
		expression->Phase2_CreateReferencesAndConnections();
	}

}
